
/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
****************************************************/
#include "p18f2523.h"
//#include "p18f2510.h"
#include <stdio.h>
#include <string.h>
#include <usart.h>
#include <eep.h>

#include "model.h"

#include "extern.h"

CMD_STATE cmd_state;

unsigned char msgbyte, ch;

unsigned char sweep, preset;


/***  INTERRUPT CODE ****/
void lowvector(void);
void ProcessInterrupt(void);


#pragma code low_vector=0x18
void interrupt_at_low_vector(void) {
  _asm GOTO lowvector _endasm
}
#pragma interruptlow lowvector
void lowvector() {
	; // do nothing 
}




#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
  _asm GOTO ProcessInterrupt _endasm
}
#pragma code /* return to the default code section */

/* DATA LINE ON INT 0   CLOCK LINE ON INT 1  */
#pragma interrupt ProcessInterrupt
void ProcessInterrupt() {
	if (PIR1bits.TMR1IF) {
		goto PROCESS_TIMER;
	}
	if (PIR1bits.RCIF) {
		goto PROCESS_UART;
	}
	if (INTCONbits.TMR0IF) {
		goto PROCESS_ALARM;
	}
	goto INT_DONE;

PROCESS_TIMER:		/* This just increment the clock */
	PIR1bits.TMR1IF = 0;
	TMR1H |= 0x80;	/* Add 32,768 to the timer */
	sec++;
	if (sec == 60) {
		sec = 0;
		min++;
	}
	if (min == 60) {
		min = 0;
		hour++;
	}
	if (hour == 24) {
		hour = 0;
	}
	goto INT_DONE;

PROCESS_UART:		/* Has a byte arrived? */
	if (RCSTAbits.FERR) {	/* UART frame error? */
		PIR1bits.RCIF = 0;	/* Clear the status */
		cmd_state = CMD_WAITING;
		ch = RCREG;;
		goto INT_DONE;
	}
	if (RCSTAbits.OERR) {	/*  UART overrrun ? */
		RCSTAbits.CREN = 0;	/* Disable it */
		RCSTAbits.CREN = 1;	/* Re enable it to reset */
		PIR1bits.RCIF = 0;		/* and start over */
		cmd_state = CMD_WAITING;
		ch = RCREG;
		goto INT_DONE;
	}
	ch = RCREG;	/* Read the byte to clear the buffer */
	if (cmd_state == CMD_COMPLETE) {	/* if we are already full, bail */
		PIR1bits.RCIF = 0;	
		goto INT_DONE;
	}

	if (ch == 0x01) msgbyte = 0;
	/* If this is the last one, flag a new message has arrived */
	if (ch == 0x04) cmd_state = CMD_COMPLETE;
		
	messagein[msgbyte] = ch;		/* Next byte into the buffer */
	msgbyte++;				/* Next position of the buffer */
	if (msgbyte == 16) msgbyte = 0;	/* Did we fill up ? */

	PIR1bits.RCIF = 0;	
	goto INT_DONE;

PROCESS_ALARM:	/* Are we sounding the alarm? */
	INTCONbits.TMR0IF = 0;	
	sweep++;
	if (sweep == 30) {	/* Every 30 steps... */
		sweep = 0;
		preset++;		/* Change the preset value  */
		if (preset == 90) preset = 0;
	}
	TMR0L = preset;	/* Which changes the freeq to a sweep */

	LATAbits.LATA1 = !LATAbits.LATA1;	/* Toggle  */
	goto INT_DONE;

INT_DONE:
	return;

}	
	