
/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
****************************************************/
#define BOR_ENABLED
#define WDT_ENABLED


#include "p18f2523.h"
#include <stdio.h>
#include <string.h>
#include <usart.h>
#include <delays.h>
#include <eep.h>


#include "extern.h"

#include "lcd.h"

#include "tools.h"

#include "controller.h"

#include "model.h"
#include "view.h"

/* A boat load of setting for the MCU config bits */
#pragma config WDT = ON
#pragma config WDTPS = 16384
#pragma config OSC = INTIO67
#pragma config BOREN = ON
#pragma config BORV = 2
#pragma config PWRT = ON
#pragma config LVP = OFF
#pragma config MCLRE = ON
#pragma config LPT1OSC = OFF
#pragma config STVREN = ON
#pragma config XINST = OFF
#pragma config DEBUG = OFF

/*	A hand full of variables that everyone shares */
unsigned char hour, min, sec;
unsigned char nowhour, nowmin, nowsec;

unsigned char pwr, rst, wdt, brn;

void ProcessIncomingMsg(void);
void GetEECode(void);

void DisplayResets(void);

void DrawMenu(void);

void PrintTime(void);

void ScanKeys(void) ;
void ProcessKeys(void);

/* This is where it all began... */
void main() {
	WDTCONbits.SWDTE = 0;
	OSCCON = 0x70;
	OSCTUNE = 0x40;
	ADCON1 = 0xff;		/* A/D is off */
	TRISA = 0x00;		/*  Port A is all output  */
	TRISC = 0x1c;		/* Buttons are  0 0 0 1 1 1 0 0  */
	INTCON2bits.RBPU = 0;	/* no pull ups on the LCD module */
	PORTA = 0;
	PIE1 = 0;
	PIE2 = 0;
	cmd_state = CMD_WAITING;		/* Our inital state between
						/* the UART ISR and the main loop  */

	InitTimer1();		/* Used to initialize the timer */

	InitializeAlarm();

	/*  19200 baud  */
	OpenUSART( USART_TX_INT_OFF & USART_RX_INT_ON &
		USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_BRGH_LOW &
		USART_CONT_RX, 25 );

	RCONbits.IPEN = 0;		/* PIC 16F compatible */
	IPR1bits.RCIP = 1;		/*  RX int enable */
	INTCONbits.PEIE = 1;	/* Preif Enable */
	ReportReset( );	
	
	INTCONbits.GIE = 1;	/* Now let interrupts occur */

	Delay1KTCYx(10);

	InitializeView();		/* Set up the view */
	InitializeModel();		/* Set up the model (system data storage */
	InitializeController();	/* set up the button interface */
/********************************
	TEST CODE
*********************************/
	DrawMenu();		/* Draw the first menu  */


	while (1) {
		ClrWdt();		/* Make sure we stay on top of things */

		ProcessController();	/*  Anyone tells us what to do? */
		ProcessModel();		/* Any new data from the network? */
		ProcessView();		/* Any changes to the display? */
		Delay10TCYx(1);	/*  Whew! That was a lot of work, need a break */
	}
}
