
/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
****************************************************/
#define BOR_ENABLED
#define WDT_ENABLED


#include "p18f2523.h"
#include <stdio.h>
#include <string.h>
#include <usart.h>
#include <eep.h>

#include "extern.h"
#include "tools.h"

#include "model.h"
#include "view.h"

CLIENT client[5];
/* Globally visible by the interrupt */
unsigned char messagein[16];
unsigned char messageout[16];
unsigned char machine;

static unsigned char clientnum;  /* the remote device 
			that sent us a message, starting from positon 0 */

void ProcessRequests(void) ;
void ProcessIncomingMsg(void) ;
void ProcessStatus(void);
void SetTime(void);
void SendMessage(void);
void ProcessTime(void);

void InitializeModel(void) {
		/* Pre build a message to send out */
	messageout[0] = 0x01;
	messageout[2] = ADDR;
	messageout[3] = 0;
	machine = 0;
}


void ProcessModel() {
	/* If the time has changed, see if we need to do a poll */
	if (sec != nowsec) {
		nowsec = sec;
		/* Build any outgoing requests */
		ProcessRequests();
		/* If there is a message to send, fire it off */
		if (messageout[3] != 0) SendMessage();
		ProcessTime();
	}
		/* Check for any inbound message */
	if (cmd_state == CMD_COMPLETE) {
		ProcessIncomingMsg();
		cmd_state = CMD_WAITING;
	}
}



void ProcessTime(void) {
	if ((hour == 21) &&
		(min == 0)  &&
		(sec == 15)) {	/* Do we need to sound the alarm? */
		if ((client[GARAGE1].status & 0x01) || 
			(client[GARAGE2].status & 0x01)	) {
			ActivateAlarm(1);
		}
	}
}

/* Based on the time, build a poll request to be fired off. */
void ProcessRequests(void) {
	if (sec == 5) {
		messageout[1] = 0x38;
		messageout[3] = 'S';
		messageout[4] = 0x04;
	}
	if (sec == 10) {
		messageout[1] = 0x39;
		messageout[3] = 'S';
		messageout[4] = 0x04;
	}
	if (sec == 15) {
		messageout[1] = 0x26;
		messageout[3] = 'S';
		messageout[4] = 0x04;
	}
	if (sec == 20) {
		messageout[1] = 0x27;
		messageout[3] = 'S';
		messageout[4] = 0x04;
	}
	if (sec == 25) {
		messageout[1] = 0x50;
		messageout[3] = 'S';
		messageout[4] = 0x04;
	}
}

/*  When a message arrives, we will have the full message, including
	SOH/ETX, in the command buffer  */
void ProcessIncomingMsg() {
	/* If it isn't for us, then ignore it.  */
	if ((messagein[1] != 0x20) &&
		(messagein[1] != ADDR)) {
		return;
	} /* [0], [1], [2]  are the headers...  */
	switch (messagein[2]) {
		case 0x26 : clientnum = LIGHT1; break;
		case 0x27 : clientnum = LIGHT2; break;
		case 0x38 : clientnum = GARAGE1; break;
		case 0x39 : clientnum = GARAGE2; break;
		case 0x50 : clientnum = 4; break;
	}
	switch (messagein[3]) {
			/* Is this a status ? */
	case 's' : ProcessStatus(); break;
			/* Is this a list of resets?  */
	case 'z' : 	client[clientnum].resets[POWERUP] = messagein[POWERUP + 4] - 0x20;
				client[clientnum].resets[WATCHDOG] = messagein[WATCHDOG + 4] - 0x20;
				client[clientnum].resets[RESET] = messagein[RESET + 4] - 0x20;
				client[clientnum].resets[BROWNOUT] = messagein[BROWNOUT + 4] - 0x20;
			break;
		/* Is this the time ? */
	case 'T' : SetTime(); break;
	}

		
}			


void ProcessStatus(void) {
	/* If it's the TIME server, get the GMT offset  */
	if (clientnum == 4) {
		client[clientnum].buffer[0] = messagein[5] - 0x20;
	}		

/*  '0' is OFF, or Closed or GPS NOT SYNC  */
	if (messagein[4] == '0') {
		client[clientnum].status &= 0xfe;
	} else {
		client[clientnum].status |= 0x01;
	}
}

void SetTime(void) {
	hour = messagein[4] - 0x20;	/* Shift down from where it has 32 added to it */
	min = messagein[5] - 0x20;
	sec = messagein[6] - 0x20;
}

void SendMessage(void) {
	unsigned char index;
	/* If we are in the process of receiving, then bail for now.
		The next pass through the controller wil catch it */
	if (cmd_state != CMD_WAITING) {
		return;
	}
	LATAbits.LATA2 = 1;	/*  Enable the RS485 line	*/
	index = 0;
	do {
		while (BusyUSART())  { ;  }
		WriteUSART(messageout[index]);
		if (messageout[index] == 0x04) break;
		index++;
	} while (1);
		
	while (BusyUSART())  { ;  }	/* Wait for it to complete */			
	LATAbits.LATA2 = 0;			/*  Disable the RS485 line  */
	messageout[3] = 0x00;
}



