/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
****************************************************/

#include "p18f2523.h"

#include <stdlib.h>

#include <usart.h>
#include <eep.h>
#include "extern.h"
#include "lcd.h"
#include "tools.h"
#include "view.h"
#include "controller.h"
#include "model.h"


/* Make all text 6 chars wide, so they are sure to overwrite the past ones that displayed */

rom char ON[] =		" On   ";
rom char OFF[] =		" Off  ";
rom char OPENED[] =	" Open ";
rom char CLOSED[] =	"Closed";

rom char PWR[]	=	"Pwr ";
rom char RST[]	=	"Rst ";
rom char WDT[]	=	"Wdt ";
rom char BRN[]	=	"Brn ";

CLIENT status[5];

static unsigned char displayOff;
static unsigned char screen, device;
char timestr[6];

void DisplayStatus(void);
void DisplayData(void);
void DisplayResets(void);
void DrawStatusTemplate(void);
void DrawResetTemplate(void);
/* Activate the backlight...  */
void ScreenOn(void) {
	displayOff = sec + 10;	/* For ten seconds */
	if (displayOff > 59) displayOff -= 60;
	LATCbits.LATC5 = 1;
}
		/* Create the time, and draw it to position 'C' */
void PrintTime(void) {
	unsigned char h;
	CursorJump(BUTTON2X);
	h = hour;
	if (h > 12) h -= 12;
	if (h == 0) h = 12;
	int2ascii(timestr,h,0);	/* Put n the hours, no leading zero */
	int2ascii(&timestr[3],min,1);	/* Minutes WITH leading zero */
	WriteString(timestr);	/* and write it  */
}

void InitializeView(void) {
	ResetDisplay();	/* Used to activate the display */
	ClearDisplay();
	HomeDisplay();
	EntryDisplay();
	ActivateDisplay();

	displayOff = sec + 10;	/* Initially the light is on */
	screen = STATUS_DISP;
	if (displayOff > 59) displayOff -= 60;
	timestr[2] = ':';	/* Pre build the time string  */
	timestr[5] = 0;
	DisplayStatus();	/* and show the model status */

}
/* let someone else (the controller) tell us to change 
	what we are displaying */
void SetMode(unsigned char x, unsigned char dev) {
	screen = x;
	device = dev;
	switch (screen) {
	case STATUS_DISP : DrawStatusTemplate() ; break;
	case STATUS_RST : DrawResetTemplate(); break;
	}
}
/*  The status screen is basically empty (no template ) */
void DrawStatusTemplate(void) {
	ClearDisplay();
}

/* But the reset screen has some text on it that is static, so we
	draw it now */
void DrawResetTemplate(void) {
	ClearDisplay();
	WriteRom(PWR,0x14);
	WriteRom(WDT,0x1e);
	WriteRom(RST,0x40);
	WriteRom(BRN,0x4a);	
}


/* Called from the main loop  */
void ProcessView(void) {	
	if (sec == displayOff) {	/* Is it tome to turn off the backlight? */
		LATCbits.LATC5 = 0;
		return;
	}
	DisplayData();	/* Display whatever data we are set to  */
}

void DisplayData(void) {
	switch (screen) {		/* Which are we showing ?  */
	case STATUS_DISP : DisplayStatus() ; break;
	case STATUS_RST : DisplayResets(); break;
	}
}

/* The status sacreen will populate itself every minute, or when
	something changes */
void DisplayStatus(void) {
	if ((client[GARAGE1].status != status[GARAGE1].status) ||
		(sec == 0)) {
		status[GARAGE1].status = client[GARAGE1].status;
		if (client[GARAGE1].status & 0x01) {
			WriteRom(OPENED,0x40);
		} else {
			WriteRom(CLOSED,0x40);
		}
	}
	if ((client[GARAGE2].status != status[GARAGE2].status) ||
		(sec == 0)) {
		status[GARAGE2].status = client[GARAGE2].status;
		if (client[GARAGE2].status & 0x01) {
			WriteRom(OPENED,0x14);
		} else {
			WriteRom(CLOSED,0x14);
		}
	}
	if ((client[LIGHT1].status != status[LIGHT1].status) ||
		(sec == 0)) {
		status[LIGHT1].status = client[LIGHT1].status;
		if (client[LIGHT1].status & 0x01) {
			WriteRom(ON,0x4a);
		} else {
			WriteRom(OFF,0x4a);
		}
	}
	if ((client[LIGHT2].status != status[LIGHT2].status) ||
		(sec == 0)) {
		status[LIGHT2].status = client[LIGHT2].status;
		if (client[LIGHT2].status & 0x01) {
			WriteRom(ON,0x1e);
		} else {
			WriteRom(OFF,0x1e);
		}
	}
}

/* The screen of resets will put the counts in the locations right
	behind the static text that was placed by the template */
void DisplayResets(void) {
	char buf[5];
	/*  If it is a new second, or there is a change...  */
	if ((client[device].resets[POWERUP] != status[device].resets[POWERUP]) ||
		(sec == 0)) {
		status[device].resets[POWERUP] = client[device].resets[POWERUP];
		itoa(status[device].resets[BROWNOUT],buf);
		CursorJump(0x18);
		WriteString(buf);
		 
	}
	if ((client[device].resets[WATCHDOG] != status[device].resets[WATCHDOG]) ||
		(sec == 0)) {
		status[device].resets[WATCHDOG] = client[device].resets[WATCHDOG];
		itoa(status[device].resets[BROWNOUT],buf);
		CursorJump(0x22);
		WriteString(buf);
	}
	if ((client[device].resets[RESET] != status[device].resets[RESET]) ||
		(sec == 0)) {
		status[device].resets[RESET] = client[device].resets[RESET];
		itoa(status[device].resets[BROWNOUT],buf);
		CursorJump(0x44);
		WriteString(buf);
	}
	if ((client[device].resets[BROWNOUT] != status[device].resets[BROWNOUT]) ||
		(sec == 0)) {
		status[device].resets[BROWNOUT] = client[device].resets[BROWNOUT];
		itoa(status[device].resets[BROWNOUT],buf);
		CursorJump(0x4e);
		WriteString(buf);
	}

}

